/**
 * 资源页面
 */

!(function (win, o) {
    'use strict';
    
    var API = SFAPI,
        log = SFLOG,
        com = SFCommon,
        cacheParam = {},
        resObj,
        resultObj = {
            success: function () {
            },
            error: function () {
            }
        },
        session,
        curTWFID = '',
        TWFID_CHECK_SPAN = 2000,
        STR_FAKE_URL = '/safeurl',
        G = SFConfig,
        isFirstL3vpn, // 是否第一个客户端L3vpn服务(非第一个客户端无法使用L3vpn资源)
        resConfig = {
            info: {},
            config: {
                unAuthData: [], // 未授权资源
                securityStrategy: [], // 安全策略数据
                enableSslTray: 0, //启用系统托盘
                autoStartClient: 0, //自动启动客户端
                securityCheckInterval: null, //安全检查间隔时间
                enableSecurityCheck: null, //启用安全检查
                isEnableComFirstDns: '', // 接入计算机使用此DNS服务器作为首选的DNS服务器
                isDisplayHost: null, //是否显示app/ip 资源的ip地址 !!@@
                otherLogin: false,
                loadFailed: false, // 加载失败，显示重新加载页面
                sslvpnNameFix: '', //sslvpn名称后缀
                ecIsExist: null
            },
            websso: [], //自动构建参数单点登录
            rc: { //原始数据 rclist > Resource, 其他数据从原始数据引用
                allData: [],
                pureData: [],
                rsType: [],
                raw: {},
                rcs: [],
                rcsHash: [], //资源数据hash, key by id
                defaults: { //默认资源 & 资源组
                    //: g_defaultRcId
                    rcId: null,
                    groupId: -1000, //指向默认资源: resConfig.rc.rcs[i]
                    rc: {},
                    group: {} // 指向默认资源组: resConfig.rc.groups[i]
                },

                // 按类型分类的资源集
                web: [],
                tcp: [],
                l3vpn: [],
                remoteApp: [],
                easyLinkHash: [],
                userBind: [],  // 主从用户名绑定资源 set of ids
                fileLock: [], // 关键文件保护数据
                unAuth: [], // 未授权资源数据
                strategyData: [], // 资源组数据
                groups: [], // 资源组数据, key by id
                groupsHash: [], // 资源组Id与对应的资源集合
                groupRcs: [],
                config: {}, // 资源配置
                other: {} // 其他资源信息
            },
            sso: { // 单点登录
                raw: {}, //原始数据
                data: [], //单点登录数据
                obj: null,
                config: { //单点登录配置
                    enable: null,
                    show_ssoconf: null,
                    show_perssoconf: null,
                    ssotimeout: null
                },
                info: { //单点登录信息
                    cert: null,         //GetCookie('SSL_CLIENT_INFO')
                    username: null,
                    userpwd: null,
                    phone_num: null,
                    group_name: null,
                    scert_info: null,
                    scert_sn: null,
                    scert_thum: null,
                    dkey_id: null
                }
            }

        },

        // 资源类别
        rcType = {
            WEB: 0, // web
            APP: 1, // tcp
            IP: 2 // l3vpn
        },

        // 资源协议
        rcProto = {
            TCP: 0, // tcp
            UDP: 1, // udp
            ICMP: 2, // icmp
            IP: 3  // ip
        },

        // 资源服务类型
        rcService = {

            HTTP: 'http',
            HTTPS: 'https',
            FILESHARE: 'fileshare',
            REMOTE_APP: 'remoteapp',
            TERMINAL_SERVICE: 'terminal service',
            DNS: 'dns',
            MAIL: 'mail',
            FTP: 'ftp'

        },



        LOGTAG = 'business: res';


    /**
     * 获取指定资源信息
     * @param {string} path
     * @param {string} defaultValue
     * @returns {*}
     */
    function _getRcValue (path, defaultValue) {
        var node = resConfig.rc.raw.Resource;
        return _getNodeValue(path, defaultValue, node);
    }

    /**
     * 获取指定配置信息
     * @param {string} path
     * @param {string} defaultValue
     * @returns {*}
     */
    function _getConfigValue (path, defaultValue) {
        var node = resConfig.config.raw.Conf;
        return _getNodeValue(path, defaultValue, node);
    }

    /**
     * 获取本机配置信息
     * @param {string} path
     * @param {string} defaultValue
     * @returns {*}
     */
    function _getLocalConfigValue (path, defaultValue) {
        var node = resConfig.config.raw.LocalConf;
        return _getNodeValue(path, defaultValue, node);
    }

    /**
     * 获取指定信息
     * @param {string} path
     * @param {string} defaultValue
     * @param {Object} node
     * @returns {*}
     */
    function _getNodeValue (path, defaultValue, node) {
        var nodes,
            i;
        if (typeof (node) === 'undefined') {
            return defaultValue;
        }
        nodes = path.split('.');
        for (i = 0; i < nodes.length; i++) {
            node = node[nodes[i]];
            if (typeof (node) === 'undefined' || node === null) {
                return defaultValue;
            }
        }
        if (is.string(node) && is.numeric(node) && (typeof (defaultValue) === 'number')) {
            node = parseFloat(node); // 根据默认值的类型, 将字符串数字转换为数字
        }
        return node;
    }

    /**
     * 资源数据初始化
     */
    function Res () {}

    // 获取初始化资源数据
    function _getStrategyAndResource (options) {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :getStrategyAndResource ');
            API.getStrategyAndResouceData({
                success: function (data) {

                    // 资源不存在时
                    if (!data || !data.data.conf || !data.data.rc || !data.data.rc.Resource) {
                        log.warn(LOGTAG,
                            'getStrategyAndResource: conf or res is null ',
                            ' token is invalid or do not set twfid to ECAgent ');
                        resConfig.config.loadFailed = true; // 加载失败
                        options.success();
                    } else {
                        log.info(LOGTAG, 'getStrategyAndResource: success ');
                        resolve(data);
                    }
                },
                error: function (err) {
                    log.error(LOGTAG, 'getStrategyAndResource: get conf or res error ', ' network request error ');
                    reject(err);
                }
            });
        });
    }

    //处理资源页面数据
    function _formatConfAndResource (data) {
        return new Promise(function (resolve, reject) {

            var info = { // 下载客户端窗口参数
                    isClose: false
                },
                message,
                strategys,
                ssoData;
            log.info(LOGTAG, 'step :formatConfAndResource  ');

            // 此处需要初始化相关策略数据（登录信息）
            // //资源数据、//登录信息、是否使用DNS、SSL VPN专线、启用系统托盘、安全检查、key登录、负载均衡
            resConfig.config.raw = data.data.conf;
            resConfig.rc.raw = data.data.rc;
            resConfig.info = data.data.conf.Conf.Other; // 登录信息
            // resConfig.other = _getConfigValue('Other', {}); // 登录信息
            resConfig.config.enableSslTray = _getConfigValue('SysTray.enable', 0);
            resConfig.config.isEnableComFirstDns = _getRcValue('Dns.dnsserver', '') !== '';
            resConfig.config.showRc = _getConfigValue('Other.enable_cs_rc_windows', 1); // 显示资源列表
            resConfig.config.enableSslvpnOnlyLine = _getConfigValue('Other.sddn_enable', 0); // SSL VPN专线
            resConfig.config.enableSecurityCheck = _getConfigValue('SecurityCheck', false) ? true : false; // 安全检查
            resConfig.config.securityCheckInterval = _getConfigValue('SecurityCheck.attribute.interval', 0);
            resConfig.config.sslvpnNameFix = _getLocalConfigValue('Session.nameFix', '');
            resConfig.info.loginName = _getConfigValue('Other.login_name', '').toString();

            //将用户名设置至缓存，通过隐藏页面同步至EasyConnect，便于下次客户端登录时显示该用户
            SF.setting.setGlobal({ key: KEY_GLOBAL_LOGIN_NAME, value: resConfig.info.loginName }, 0);

            //1为系统托盘，0为悬浮窗托盘
            SF.setting.setGlobal({ key: KEY_GLOBAL_TRAY_TYPE, value: resConfig.config.enableSslTray}, 0);
            resConfig.info.isReLogin = _getConfigValue('Other.isRelogin', 0);
            resConfig.info.lastLoginTime = _getConfigValue('Other.RelogTimeLast', 0);
            resConfig.info.lastLoginIp = _getConfigValue('Other.RelogIPLast', '');
            resConfig.config.httpPort = _getConfigValue('WebHttpEnable.httpPort', 0);
            resConfig.config.enable = _getConfigValue('WebHttpEnable.enable', 0);


            //安全检查，需弹出客户端下载的窗口
            if (resConfig.config.enableSecurityCheck && !SFConfig.isExistEC) {
                log.info('installClient:', 'Security check enabled, Must install client.');
                SFCommon.installClient({isClose: false});
                return;
            }

            /*
            //启用系统托盘，需弹出客户端下载的窗口
            if (resConfig.config.enableSslTray && !SFConfig.isExistEC) {
                log.info('installClient:', 'System tray enabled Must install client.');
                SFCommon.installClient({isClose: false});
                return;
            }
            */

            //VPN虚拟专线且未安装客户端，需弹出客户端下载的窗口
            if (resConfig.config.enableSslvpnOnlyLine && !SFConfig.isExistEC) {
                log.info('installClient:', 'VPN Line enabled, Must install client.');
                SFCommon.installClient({isClose: false});
                return;
            }

            //内网DNS且未安装客户端，需弹出客户端下载的窗口
            if (resConfig.config.isEnableComFirstDns && !SFConfig.isExistEC) {
                log.info('installClient:', 'DNS enabled, Must install client.');
                SFCommon.installClient({isClose: false});
                return;
            }


            //websso
            resConfig.websso = _buildArray('Resource.WebSsoInfos.webssoinfo', resConfig.rc.raw);
            resConfig.websso.forEach(function (ssoinfo) {
                ssoinfo.args = _buildArray('args', ssoinfo);
                ssoinfo.args.forEach(function (arg) {
                    arg.value = _htmlDecode(arg.value);
                });
                ssoinfo.rcid = Number(ssoinfo.rcid);
            });

            ssoData = _getRcValue('Sso.data', '');
            if (ssoData) {
                ssoData.split(';').forEach(function (val) {
                    var kv = val.split(':');
                    if (!resConfig.sso.data) {
                        resConfig.sso.data = {};
                    }
                    resConfig.sso.data[kv[0]] = {data: kv[1]};
                    if (is.set(kv[2]) && kv[2]) {
                        resConfig.sso.data[kv[0]].mark = kv[2];
                    }
                });
            }
            strategys = _buildArray('Conf.SecurityCheck.strategies.strategy', resConfig.config.raw);
            strategys.forEach(function (strategy) {
                var id = strategy.id;
                resConfig.config.securityStrategy[id] = strategy;
            });

            resConfig.sso.raw = resConfig.rc.raw.Resource.Sso;
            resConfig.rc.groups = _buildArray('Resource.RcGroups.Group', resConfig.rc.raw);
            resConfig.rc.rcs = _buildArray('Resource.Rcs.Rc', resConfig.rc.raw);
            resConfig.config.ecIsExist = SFConfig.isExistEC;// 获取是否安装客户端
            resolve();
        });
        log.info(LOGTAG, 'format conf data And resource data success ');
    }

    function _syncLoginInfo () {
        return new Promise(function (resolve, reject) {
            var loginName = resConfig.info.loginName,
                vpnInfo = SF.vpnInfo.createInstance(),
                model;
            if (loginName) {
                model = vpnInfo.get();
                model.loginName = loginName;
                vpnInfo.save(model);
            } 
            resolve();
        });
    }

    function _initParam () { 
        var keys = [],
            arr = [];
        keys.push(KEY_GLOBAL_TWFIDTMP);
        keys.push(KEY_GLOBAL_EC_TOKEN);
        keys.push(KEY_GLOBAL_FROM_URL);
        keys.push(KEY_GLOBAL_VPN_PORT);
        keys.push(KEY_GLOBAL_VPN_IP);
        keys.push(KEY_GLOBAL_EC_PORT);
        keys.push(KEY_GLOBAL_EC_GUID);
        keys.push(KEY_GLOBAL_VPN_URL);
        arr = SF.setting.getGlobal(keys, []);
        cacheParam.twfIDTmp = arr[0];
        cacheParam.token = arr[1];
        cacheParam.fromURL = arr[2];
        cacheParam.vpnPort = arr[3];
        cacheParam.vpnIP = arr[4];
        cacheParam.port = arr[5];
        cacheParam.guid = arr[6];
        cacheParam.vpnURL = arr[7];
        SF.setting.delNoCacheField([KEY_GLOBAL_TWFID, KEY_GLOBAL_EC_TOKEN]);
    }


    //主要用于CS登录模式下，portal页面无须定时请求ecagent从而获取到session切换，只需隐藏页面切换session后，当前页面通过定时器对比即可
    function _checkTWFIDChange () {
        var twfID;
        if (!SFCommon.isWeb()) { 
            return;
        }
        
        twfID = session.getTWFID();
        if (twfID !== curTWFID) {
            SFAPI.cookieSaveLocal({
                success: function () {
                    log.info(LOGTAG, 'switch session,old: ' + curTWFID + ',new: ' + twfID);
                    curTWFID = twfID;
                }
            });
        } else { 
            
            //定时检测是否存在session切换
            setTimeout(function () {
                _checkTWFIDChange();
            }, TWFID_CHECK_SPAN);
        }
    }

    function _checkECAgent () {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step : checkECAgent');
            if (G.isExistEC === null) {
                log.info(LOGTAG, 'checkECAgent ECAgent not Exist,should be detect');
                API.checkECAgent({
                    success: function () {
                        log.info(LOGTAG, ' checkECAgent success ');
                        resolve();
                    },
                    error: function () {
                        resolve();
                    }
                });
            } else {
                log.info(LOGTAG, ' checkECAgent success ');
                resolve();
            }
        });
    }

    // 初始化ECAgent
    function _initECAgent (options) {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step : initECAgent');

            API.initECAgent({
                success: function (data) {
                    log.info(LOGTAG, 'init ECAgent success');
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, 'Failed to init ECAgent', 'error is %s ', com.getMessage(err.code));
                    reject(err);
                }
            });
        });
    }

    //同步登录session到客户端
    function _syncTWFID (options) {
        log.info(LOGTAG, 'step :syncTWFID ');
        return new Promise(function (resolve, reject) {
            var id = session.getTWFID();

            if (!id) {
                log.warn(LOGTAG, 'Can not get TWFID by cache:twfIDTmp', 'first auth do not return twfID, continue using old twfid');
            }

            API.setTWFID({
                twfID: id,
                isForce: true,
                success: function () {
                    log.info(LOGTAG, ' sync twfid success ');
                    resolve();
                },
                error: function () {
                    log.error(LOGTAG, 'sync twfid error', 'network request error');
                    reject('Failed to sync TWFID');
                }
            });
        });
    }

    /**
     * 启动服务
     * @return {*}
     */
    function _startService () {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :startService ');
            API.startService({
                success: function (data) {
                    log.info(LOGTAG, ' start service success ');
                    isFirstL3vpn = (data.data.l3vpn !== G.SERVICE_STATE.STATUS_NEED_NOT_SERVICE);
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, ' start service error ', ' network request error or service do not start ');
                    reject(err);
                }
            });
        });
    }

    /**
     * 查询服务状态
     * 用于继续上次的查询, 并且判断当前L3vpn资源是否支持
     * @return {*}
     */
    function _queryService () {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :queryService ');
            API.queryService({
                success: function (data) {
                    log.info(LOGTAG, ' query service success ');
                    isFirstL3vpn = (data.data.l3vpn !== G.SERVICE_STATE.STATUS_NEED_NOT_SERVICE);
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, ' query service error ', ' network request error or service do not query ');
                    reject(err);
                }
            });
        });
    }


    /**
     * 启动托盘
     * @return {*}
     */
    function _startEcTray () {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :startEcTray ');
            API.startECTray({
                success: function (data) {
                    log.info(LOGTAG, 'start tray success');
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, 'Failed to start the tray', ' network request error ');
                    reject(err);
                }
            });
        });
    }

    /**
     * 设置服务器地址（告知客户端服务器地址）
     * @param {*} options
     * @return {*}
     */
    function _settingServerAddr (options) {
        var addr;
        addr = cacheParam.vpnIP + ' ' + cacheParam.vpnPort;

        log.info(LOGTAG, 'step :set server address ');
        return new Promise(function (resolve, reject) {
            SFAPI.doConfigEC({
                key: 'S_SERVADDR',
                params: {
                    address: addr
                },
                success: function (data) {
                    log.info(LOGTAG, 'set Server Addr success ');
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, ' set server address failed ', ' network requset error ');
                    reject(err);
                }
            });
        });
    }

    /**
     * 设置来源URL
     * @param {*} options
     * @return {*}
     */
    function _settingFrom (options) {

        return new Promise(function (resolve, reject) {
            var type = '';
            log.info(LOGTAG, 'step :set from url  ');
            if (!options || !options.fromAddr) {

                // 来源地址可以为空, 为空不设置, 打警告日志后继续.
                log.warn(LOGTAG, 'set from url failed', 'settingFrom: fromURL is empty ,continue ');
                resolve();
                return;
            }
            if (G.systemType === G.CLIENT_TYPE.WINDOWS) {
                type = 'C_LOGINADDR';
            } else {
                type = 'S_LOGINADDR';
            }
            SFAPI.doConfigEC({
                key: type,
                params: {
                    vpnLine: options.fromAddr
                },
                success: function (data) {
                    log.info(LOGTAG, 'Set source url success');
                    resolve(data);// :
                },
                error: function (err) {
                    log.error(LOGTAG, 'Failed Set the source url', ' network request error ');
                    reject(err);
                }
            });
        });
    }

    // 设置浏览器类型
    function _settingBrowserType (options) {
        log.info(LOGTAG, 'step :set browser type ');
        if (!options || typeof options.browserType !== 'undefined') {
            log.info(LOGTAG, 'settingBrowserType: check params error ', ' current param is invalid ');
            return;
        }
        return new Promise(function (resolve, reject) {
            SFAPI.doConfigEC({
                key: 'S_BROWSER',
                params: {
                    browserType: options.browserType
                },
                success: function (data) {
                    log.info(LOGTAG, 'set browser type success  ');
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, 'set  browser type failed ', 'network request error ');
                    reject(err);
                }
            });
        });
    }

    /**
     * 设置证书
     * @return {*}
     */
    function _setCertInterface () {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :set sso cert ');
            if (resConfig.sso.info.cert) {
                SFAPI.doConfigEC({ // 设置证书
                    key: 'C_CERTINFO',
                    params: {
                        cert: resConfig.sso.info.cert
                    },
                    success: function (data) {
                        log.info(LOGTAG, 'set cert info: success ');
                        resolve(data);
                    },
                    error: function (err) {
                        log.error(LOGTAG, 'set cert failed', ' network request error ');
                        reject(err);
                    }
                });
            } else {
                log.info(LOGTAG, 'No cert info, pass,success');
                resolve();
            }
        });
    }


    /**
     * 查询登录状态
     * @param {*} options
     * @return {*}
     */
    function _queryLoginStatus (options) {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :queryLoginStatu');
            API.checkReLogin({
                success: function (data) {
                    log.info('BUSINESS:queryLoginStatu', 'query login status');
                    if (!data || typeof data.code === 'undefined') {
                        return new Promise(function () { // 阻断
                            resConfig.config.loadFailed = true; // 加载失败
                            options.success(); // 显示“打开资源页面”按钮页面
                        });
                    }
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, 'queryLoginStatu:Query login status failed', ' cause by network request error   ');
                    reject(err);
                }
            });
        });
    }

    function _checkReLogin (data, options) {

        // 检查登录状态根据不同状态跳转分支
        var errCode = {
            notLogin: 0, // 未登录
            selfLogin: 1,   // 已登录
            otherLogin: 2 // 客户端已登录，本地没有session信息
        };

        if (!G.isExistEC) {
            resConfig.config.isFirstLogin = true;
            return false;
        }

        log.info(LOGTAG, 'step :checkReLogin ');
        if (data.code === errCode.notLogin) {
            log.info(LOGTAG, 'first login,current status is ' + data.code + 'no user login');
            resConfig.config.isFirstLogin = true; // 标记第一次登录
            return _syncTWFID(); // 同步TWFID（session）
        } else if (data.code === errCode.selfLogin) {
            log.info(LOGTAG, 'relogin,current status is ' + data.code + ', self login');
            return _syncTWFID(); // 同步TWFID（session）
        } else if (data.code === errCode.otherLogin) {
            log.info(LOGTAG, 'relogin,other login,can not excute next temp（syncTWFID）,current status is  ', data.code);
            resConfig.config.otherLogin = true;
            return new Promise(function () { // 阻断
                options.success(); // 显示“打开资源页面”按钮页面
            });
        }
    }

    // 检查是否启用代理
    function _checkProxy (options) {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :checkProxy ');
            API.checkProxy({
                success: function (data) {
                    var isEnableProxy;
                    if (data && data.code === 0 && data.result && data.enable === 0) {
                        log.info(LOGTAG, 'enable proxy ');
                        isEnableProxy = true;
                        resolve(data); // data:获取数据，true：表示启用了代理
                    } else {
                        log.info(LOGTAG, 'disable proxy');
                        isEnableProxy = false;
                        resolve(data);
                    }
                    log.info(LOGTAG, 'success');

                    // 检测到启用代理，调用测试代理接口，检测是否可用
                    resConfig.info.isUseIeProxy = isEnableProxy ? true : false; // 已启用代理
                },
                error: function (err) {
                    log.error(LOGTAG, ' checkProxy: error is  ' +
                        com.getMessage(err.code),
                    'network request error or return error code ');
                    reject(err);
                }
            });
        });
    }

    // 获取未授权资源
    function _getUnAuthRes (callBack) {
        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step :getUnAuthRes ');
            API.getUnAuthRes({
                success: function (data) {
                    log.info(LOGTAG, 'getUnAuthRes: success ');
                    if (data && data.data) {
                        resConfig.rc.strategyData = data.data.rc;
                    }
                    _queryUnAuthResTimer(callBack);
                    resolve(data);
                },
                error: function (err) {
                    log.error(LOGTAG, 'Failed to acquire unauthorized resource', ' error is %s ', err.msg);
                    reject(err);
                }
            });
        });
    }

    //启用定时检测未授权资源
    function _queryUnAuthResTimer (callBack) {
        var oneMinute = 60000;
        log.info(LOGTAG, 'step: check unauth resource security config ');
        if (!resConfig.config.enableSecurityCheck) {

            //code宏定义
            log.info(LOGTAG, 'queryUnAuthResTimer:do not ues unauth SecurityCheck ');
        }

        //启用定时检查
        if (resConfig.config.securityCheckInterval) {
            log.info(LOGTAG, 'step: start checking unauth resource circle ');

            //启用
            setInterval(function () {
                API.getUnAuthRes({
                    success: function (data) {
                        log.info(LOGTAG, 'queryUnAuthResTimer: get unauth data ');
                        if (callBack) {
                            callBack(data.data.rc);
                        }
                    },
                    error: function (err) {
                        log.info(LOGTAG, 'queryUnAuthResTimer:Failed to acquire unauthorized resource', ' network request error ');
                    }
                });
            }, oneMinute * resConfig.config.securityCheckInterval);
        }
    }

    // 便于调用，此处集中：设置浏览器类型、来源、客户端地址、证书四个接口
    function _syncBaseInfoToClient (type) {


        var currBrowserType,
            fromAddr = cacheParam.fromURL,
            urlInfo = _parseUrl(),
            serverAddr = urlInfo.host + ' ' + urlInfo.port;
        if (com.isWeb()) {
            currBrowserType = is.Browser();
        } else {
            currBrowserType = 'default';
        }
        SF.setting.setGlobal({ key: KEY_GLOBAL_BROWSER_TYPE, value: currBrowserType}, 0);
        if (type === 'before') {
            log.debug(LOGTAG,
                'step :syncBaseInfoToClient before start service,need _settingBrowserType、_settingServerAddr、_setCertInterface  ');
            return Promise.all([
                _settingBrowserType({browserType: currBrowserType}),
                _settingServerAddr({address: serverAddr}),

                // temp-set
                _setCertInterface()
            ], function (resolve, reject) {
                resolve(arguments); // 将所有数据返回
            });
        }
        log.debug(LOGTAG,
            'step :syncBaseInfoToClient after start service,need _settingBrowserType、_settingFrom、_settingServerAddr、_setCertInterface  ');
        return Promise.all([
            _settingBrowserType({browserType: currBrowserType}),

            // temp-set
            _settingFrom({fromAddr: fromAddr}),
            _settingServerAddr({address: serverAddr}),

            // temp-set
            _setCertInterface()
        ], function (resolve, reject) {
            resolve(arguments); // 将所有数据返回
        });
    }

    /**
     * 解析所有资源数据
     * 移除未授权组，修改host为数组类型，移除Web全网与泛域名资源
     * @param {object} data
     */
    function _initRsData (data) {
        var currRsType = [],
            currRsData = [],
            forCurrData = [],
            grpID,
            rcID,
            rsData = [],
            rsPureData = [],
            webRsArr = [],
            l3vpnRsArr = [],
            tcpRsArr = [],
            notAuth = -2, // 未授权资源组，默认为-2，当 存在时赋值下标
            alternativeDnsID = -97, // 备选 DNS ID
            firstChoiceDnsID = -98, // 首选 DNS ID
            webAllNetRsID = -99, // web全网资源ID
            genericDomainRsID = -100, // 泛域名全网资源ID
            hideRs = [genericDomainRsID, webAllNetRsID, firstChoiceDnsID, alternativeDnsID], // 需要隐藏的资源
            i, m, j, k, // for循环应用
            easyLinkArr,
            fileLockStr,
            pair,
            host,
            webAllNetRsName = 'web全网资源(或服务)',  // web全网资源默认名字
            webL3vpnRsName = 'L3VPN全网资源(或服务)', // webL3VPN资源默认名字
            balanceIDStar = -1000,
            balanceIDEnd = -30000,
            userBindBit = 12; // 1100

        log.info(LOGTAG, 'step : init res data ');
        if (!(resConfig.rc.rcs && resConfig.rc.groups)) {
            log.warn(LOGTAG, '_initRsData(). Initialize resource data exceptions', ' no resource data');
            return;
        }
        currRsType = resConfig.rc.groups;
        currRsData = resConfig.rc.rcs;
        rsData = currRsType;
        resConfig.rc.config.exitLoadBalance = false; // 标记默认不存在负载均衡资源

        for (i = 0; i < currRsData.length; i++) {
            forCurrData = currRsData[i];
            forCurrData.id = Number(forCurrData.id);
            grpID = parseInt(forCurrData.rc_grp_id, 10);
            rcID = parseInt(forCurrData.id, 10);
            if (hideRs.indexOf(forCurrData.id) >= 0 || (parseInt(forCurrData.attr, 10) & 1 << 0)) { // 需要隐藏的元素
                forCurrData.isBalance = false;
                if (rcID === webAllNetRsID && forCurrData.name === webAllNetRsName) { // 控制台可以修改名称，所以只对默认名称翻译
                    resConfig.rc.config.exitAllNet = true; // 标记全网资源
                    forCurrData.name = tr(webAllNetRsName);
                }
                continue;
            }

            if (rcID === 0 && forCurrData.name === webL3vpnRsName) { // 控制台可以修改名称，所以只对默认名称翻译
                forCurrData.name = tr(webL3vpnRsName);
            }
            typeFor:
            for (m = 0; m < currRsType.length; m++) {
                if (Number(forCurrData.rc_grp_id) === Number(currRsType[m].id)) {
                    if (!rsData[m].rcs) {
                        rsData[m].rcs = [];
                    }
                    forCurrData.rc_logo = '/por/svpnrcico/' + forCurrData.rc_logo; // 增加图片路径
                    forCurrData.type = Number(forCurrData.type);
                    forCurrData.proto = Number(forCurrData.proto);
                    forCurrData.id = Number(forCurrData.id);
                    forCurrData.enable_disguise = Number(forCurrData.enable_disguise);
                    rsData[m].load_balance = Number(rsData[m].load_balance);
                    forCurrData.svc = forCurrData.svc.toLowerCase(); // 全部转换为小写

                    // 前3个字符是ftp 则为ftp类型, 原因: ftp 在rclist中是svc="FTP (port/pasv mode)"
                    // eslint-disable-next-line
                    if (forCurrData.svc.substr(0, 3) === rcService.FTP) {
                        forCurrData.svc = rcService.FTP;
                    }


                    forCurrData.attr = Number(forCurrData.attr);
                    forCurrData.app_path = _htmlDecode(forCurrData.app_path);

                    // 把host中单一的%符号替换为%25, 因为单一的%是非法的url地址;
                    host = _htmlDecode(forCurrData.host).replace(/%([^a-fA-F0-9]|$)/g, '%25$1');
                    forCurrData.ips = _buildIpArray(host);
                    forCurrData.ports = _buildPortArray(forCurrData.port);

                    forCurrData.addr_link = [];
                    if (forCurrData.type === 0) {
                        forCurrData.addr_link.push(host);
                    } else if (forCurrData.host) {
                        forCurrData = _rsIpBuild(forCurrData);
                    } else {
                        forCurrData.addr_link[0] = '';
                    }

                    if (rsData[m].load_balance && forCurrData.id < 0) {
                        resConfig.rc.config.exitLoadBalance = true;
                        forCurrData.isBalance = true;
                    } else {
                        forCurrData.isBalance = false;
                    }

                    if (forCurrData.svc === rcService.REMOTE_APP) {//建立remote app集合
                        forCurrData.isRemoteapp = true;
                        resConfig.rc.remoteApp.push(forCurrData);
                    }

                    if (forCurrData.attr & userBindBit) { //主从用户名绑定, 处理非win是否支持
                        resConfig.rc.userBind.push(rcID);
                    }

                    if (!is.set(resConfig.rc.groupRcs[grpID])) { //添加资源Id到资源组Id对应的资源集合
                        resConfig.rc.groupRcs[grpID] = [];
                    }
                    resConfig.rc.groupRcs[grpID].push(forCurrData.id);

                    if (!is.set(resConfig.rc.groupsHash[grpID])) {
                        resConfig.rc.groupsHash[grpID] = currRsType[m]; // 属性转换为数字
                    }

                    if (rcID !== webAllNetRsID && !(forCurrData.attr & 1)) {
                        resConfig.rc.groupsHash[grpID].display = true; // 资源组可显示标记
                    }

                    if (rcID <= balanceIDStar && rcID > balanceIDEnd) {

                        //存在负载均衡资源
                        resConfig.info.haveBalanceResource = true;
                    }

                    //group['attr'] = parseInt(group['attr']);
                    //resConfig.rc.groupsHash[id] = group;

                    // todo
                    // eslint-disable-next-line
                    switch (parseInt(forCurrData['type'])) {
                        case rcType.WEB:
                            resConfig.rc.web.push(forCurrData);
                            break;

                        case rcType.IP:
                            resConfig.rc.l3vpn.push(forCurrData);
                            break;

                        case rcType.APP:
                            resConfig.rc.tcp.push(forCurrData);
                            break;
                    }
                    resConfig.rc.rcsHash[rcID] = forCurrData;
                    rsPureData.push(forCurrData);
                    rsData[m].rcs.push(forCurrData);
                    break typeFor;
                }
            }
        }

        resConfig.rc.defaults.rcId = _getRcValue('Other.defaultRcId', 0);
        if (resConfig.rc.defaults.rcId) {
            resConfig.rc.defaults.rc = resConfig.rc.rcsHash[resConfig.rc.defaults.rcId];
        }

        for (j = rsData.length - 1; j >= 0; j--) {
            if (!currRsType[j].rcs) { // 移除没有资源的资源组（包含未授权资源组）
                currRsType.splice(j, 1);
            }
        }

        resConfig.rc.allData = rsData;
        resConfig.rc.pureData = rsPureData;
        resConfig.rc.rsType = currRsType;

        //建立easyLinkHash by id
        easyLinkArr = _buildArray('Resource.Easylink.ElnkRc', resConfig.rc.raw);
        easyLinkArr.forEach(function (rc) {
            var addr = rc.MapAddr,
                url;
            if (!is.set(rc.Id)) {
                return false;
            }

            if (!is.empty(addr)) {
                if (rc.Mode === '0') {

                    //下发端口
                    // eslint-disable-next-line
                    url = format('https://{0}:{1}', _parseUrl().host, addr);

                    //与当前vpn地址组成新的URL
                    rc.MapAddr = url;
                } else {

                    //下发域名
                    if (addr.match(/https?/i) === null) {
                        addr = _htmlDecode(addr);
                        addr = 'https://' + addr;
                        rc.MapAddr = addr;
                    }
                }
                resConfig.rc.easyLinkHash[rc.Id] = rc;
            }
        });

        //关键文件保护
        fileLockStr = _getRcValue('FileLock.data', '').trim();
        pair = [];
        if (fileLockStr) {
            pair = fileLockStr.split(';');
            pair.forEach(function (v) {
                var keyValues;
                v = v.trim();
                if (v !== '') {
                    keyValues = v.split(':');
                    if (is.set(keyValues[0])) {
                        resConfig.rc.fileLock.push(keyValues[0]);
                    }
                }
            });
        }

        // eslint-disable-next-line
        var hasTcp = resConfig.rc.tcp.length ? 1 : 0,
            hasRemote = resConfig.rc.remoteApp.length ? 1 : 0;
        
        SF.setting.setGlobal({ key: KEY_GLOBAL_HAS_TCP_RESOURCE, value: hasTcp }, 0);
        SF.setting.setGlobal({ key: KEY_GLOBAL_HAS_REMOTE_APP, value: hasRemote }, 0);

        log.debug(LOGTAG, 'initRsData: success');
    }

    function _rsIpBuild (rsItem) {
        var k;
        for (k = 0; k < rsItem.ips.length; k++) {
            rsItem.ips[k].host_to = rsItem.ips[k].host_to || '';
            if (rsItem.ports[k].port_to === rsItem.ports[k].port_from) {
                if (rsItem.ips[k].host_to) {
                    rsItem.ips[k].host_to = '-' + rsItem.ips[k].host_to;
                    rsItem.isAddrRange = true;
                }
                rsItem.addr_link.push(rsItem.ips[k].host_from + (rsItem.ips[k].host_to));
            } else {
                if (rsItem.ips[k].host_to) {
                    rsItem.ips[k].host_to = rsItem.ips[k].host_to + ':';
                }
                rsItem.addr_link.push(rsItem.ips[k].host_from + ':' +
                        rsItem.ports[k].port_from + '-' + (rsItem.ips[k].host_to) + rsItem.ports[k].port_to);
                rsItem.isAddrRange = true;
            }
        }
        return rsItem;
    }

    function _syncInfoForLocalPage () {
        return new Promise(function (resolve, reject) {

            var loginAuthConfig = SF.setting.getGlobal(KEY_GLOBAL_INIT_GET_INIT_CONFIG_DATA, false),
                enableSavePwd,
                enableAutoLogin;

            log.assert(loginAuthConfig, 'Can not get loginAuthConfig data from getGlobal.');

            enableAutoLogin = loginAuthConfig.enableAutoLogin;
            enableSavePwd = loginAuthConfig.enableSavePwd;

            API.syncInfoForLocalPage({
                info: {
                    enableAutoLogin: enableAutoLogin,
                    enableSavePwd: enableSavePwd,
                    svpnID: _getConfigValue('Service.SvpnID', ''),
                    hasTcpResource: (resConfig.rc.tcp.length ? 1 : 0),
                    hasRemoteApp: (resConfig.rc.remoteApp.length ? 1 : 0),
                    enableHtp: _getConfigValue('Htp.enable', 0),
                    enableScache: _getConfigValue('SCache.enable', 0),
                    enableWebOpt: _getConfigValue('WebOpt.enable', 0),
                    webOptDevice: _getConfigValue('WebOpt.device', 0),
                    showRc: resConfig.config.showRc
                },
                success: function () {
                    log.debug(LOGTAG, 'step :syncInfoForLocalPage success, sync data for local page');
                    resolve();
                },
                error: function (err) {
                    log.error(LOGTAG, 'step :syncInfoForLocalPage error', 'error is %s ', err.msg);
                    reject();
                }
            });
        });
    }

    //检测是否需要强制安装客户端
    function _checkMustInstallClient (options) {

        return new Promise(function (resolve, reject) {
            log.info(LOGTAG, 'step : checkMustInstallClient ');

            // 默认不需要强制安装客户端
            resConfig.config.mustInstallClient = false;

            // 判断是否需要安装客户端
            if (resConfig.config.isEnableComFirstDns || resConfig.config.enableSslvpnOnlyLine ||

                // resConfig.config.enableSslTray ||
                resConfig.config.enableSecurityCheck || resConfig.rc.config.exitLoadBalance) {
                log.info(LOGTAG, ' must install client,can not excute next step ');

                resConfig.config.mustInstallClient = true; // 必须安装客户端 todo_temp_mq
                options.pending();
                resolve();
            } else {
                log.info(LOGTAG, ' do not need install ec client ,continue next step ');
                resolve();
            }
        });
    }

    Res.prototype = {
        init: function (options) {
            var _this = this;
            options = avalon.mix(resultObj, options);
            session = SF.session.createInstance();

            //资源页面记录当前页面打开的twfid，用于检查是否隐藏页面切换了session，从而无须cs模式通过query logout切换本地cookie
            curTWFID = session.getTWFID();

            //定时检测隐藏页面是否发生了session切换
            _checkTWFIDChange();
            _initParam();

            //SFCommon.installClient = common.auth.installClient;

            // 探测ecagent
            _checkECAgent().then(function () {
                
                if (G.isExistEC) {
                    
                    // 初始化ECAgent
                    return _initECAgent();
                }
            })
                .then(function () {

                    // 查询登录状
                    return _queryLoginStatus();
                })
                .then(function (data) {

                    //检测状态，跳转不同分支
                    if (G.isExistEC) {
                        return _checkReLogin(data, options);
                    }

                    // 纯web
                    // 每次刷新都是第一次
                    resConfig.config.isFirstLogin = true;
                })
                .then(function () {

                    if (G.isExistEC) {

                        // 检测代理
                        return _checkProxy();
                    }
                })
                .then(function () {

                    // 获取策略与资源
                    return _getStrategyAndResource(options);
                })
                .then(function (data) {

                    //格式化数据
                    return _formatConfAndResource(data);
                })
                .then(function () { 
                    if (!com.isWeb() && resConfig.config.isFirstLogin) { 
                        
                        //容器环境下需要同步登陆信息
                        return _syncLoginInfo();
                    }
                })
                .then(function () {

                    // 格式化sso数据
                    _initSsoData();

                    // 初始化sso组件
                    // _initSso();

                    // 启动服务前，设置浏览器类型、服务端地址、证书
                    if (G.isExistEC && resConfig.config.isFirstLogin) {
                        return _syncBaseInfoToClient('before');
                    }
                })
                .then(function () {

                    if (G.isExistEC && resConfig.config.isFirstLogin) {

                        // 启动服务
                        return _startService();
                    } else if (G.isExistEC && !resConfig.config.isFirstLogin) {
                        return _queryService();
                    }

                })
                .then(function () {
                    if (G.isExistEC && resConfig.config.isFirstLogin) {

                        // 启动托盘
                        return _startEcTray();
                    }
                })
                .then(function () {

                    if (G.isExistEC && resConfig.config.isFirstLogin) {

                        // 启动服务后，数据可能存在丢失，重新设置浏览器类型、来源、客户端地址、证书
                        return _syncBaseInfoToClient();
                    }
                })

                /*
                .then(function () {

                    // 判断是否存在默认跳转页面
                    if (resConfig.rc.defaults.rcId && resConfig.config.isFirstLogin) {
                        log.info(LOGTAG, 'checkDefautlPage: exist default rc data , can not excute next step ');

                        // eslint-disable-next-line
                        return Promise(function () {
                            options.success();
                        });
                    }
                })
                */
                .then(function () {
                    if (G.isExistEC) {

                        // 获取未授权资源并判断是否存在安全检查，并设置定时安全检查时间
                        return _getUnAuthRes(options.onGetUnAuthData);
                    }
                })
                .then(function (data) {

                    // 初始化资源数据
                    _initRsData();
                    _this.rsInit = true;

                    //检查是否需要强制安装客户端
                    return _checkMustInstallClient(options);
                })
                .then(function () {
                    if (G.isExistEC && resConfig.config.isFirstLogin) {

                        // 同步数据, 用于本地页面(系统设置)使用
                        return _syncInfoForLocalPage();
                    }
                })
                .then(function () {

                    // 上述流程执行完后回调成功
                    log.info(LOGTAG, 'last step: invoke success callback ');
                    options.success();
                }) // eslint-disable-next-line dot-notation
                ['catch'](function (e) {

                    log.error(LOGTAG, 'catch step: proccess error  ', ' occur a error in proccess,error is %s ', e.message);
                    resConfig.config.loadFailed = true; // 加载失败
                    return new Promise(function () { // 阻断
                        log.warn(LOGTAG, 'failed ', 'proccess catch: Failed to initialize data: ' + e);
                        options.success(); // 显示“打开资源页面”按钮页面
                    });
                });
        },
        updateBalanceRes: function (options) {
            if (!options) {
                return;
            }
            SFAPI.updateBalanceRes({
                rcName: options.name,
                grpID: options.rc_grp_id,
                secID: options.selectid,
                autoID: options.id,
                success: function (data) {
                    log.info(LOGTAG, 'updateBalanceRes: The load balancing update was successful');
                    options.success();
                },
                error: function (error) {
                    log.error(LOGTAG, 'updateBalanceRes error', 'error is %s', error.msg);
                    options.error();
                }

            });
        },
        rsDataHintManage: function (rsDataItem) {
            var lineBreak = ' \n', // 换行符
                circleSymbol = '● ',
                fileLockStr, // 提示符号
                RC_MATCH_NUMBER = 0x0c,
                useUb,
                bSsoConfigStr;
            if (!is.Win()) {

                //主从用户名绑定资源
                if (rsDataItem.attr & RC_MATCH_NUMBER) {
                    if (resConfig.rc.rcsHash[rsDataItem.id] && rsDataItem.type !== rcType.WEB) { // 非web资源不支持
                        rsDataItem.msg += circleSymbol + tr('主从用户名绑定资源：当前系统不支持，请使用Windows访问') + lineBreak;
                    }
                }

                //关键文件保护
                fileLockStr = _getRcValue('FileLock.data', '').trim();
                if (fileLockStr) {
                    fileLockStr.split(';').forEach(function (v) {
                        var keyValues;
                        v = v.trim();
                        if (v !== '') {
                            keyValues = v.split(':');
                            if (is.set(keyValues[0]) && rsDataItem.id === Number(keyValues[0])) {
                                rsDataItem.state = false;
                                rsDataItem.msg += circleSymbol + tr('关键文件保护资源：当前系统不支持，请使用Windows访问') + lineBreak;
                            }
                        }
                    });
                }
            }
            if (is.Linux) {
                if (rsDataItem.svc === rcService.REMOTE_APP) { //远程应用资源
                    rsDataItem.state = false;
                    rsDataItem.msg += circleSymbol + tr('远程应用资源：当前系统不支持，请使用Windows访问') + lineBreak;
                }
            }

            // eslint-disable-next-line
            useUb = rsDataItem.attr & 1 << 3;
            if (rsDataItem !== null && useUb) {
                if (resConfig.sso.data.length) {
                    bSsoConfigStr = resConfig.sso.data[rsDataItem.id];
                }
                if (!bSsoConfigStr) {
                    rsDataItem.state = false;
                    rsDataItem.msg += circleSymbol + tr('该资源启用了单点登录的主从用户名绑定,但未录制单点登录信息,不允许访问,请联系管理员!') + lineBreak;
                }
            }

            if (rsDataItem.type === rcType.IP) { // 对L3VPN单独处理
                if (!isFirstL3vpn) { // 非首个客户端的l3vpn服务
                    rsDataItem.state = false;
                    rsDataItem.msg += circleSymbol + tr('L3VPN无法使用，当前登录了多个VPN，只有首个登录的VPN才可以使用L3VPN；') + lineBreak;
                }
                if (resConfig.info.isUseIeProxy) {
                    rsDataItem.msg += circleSymbol + tr('该资源无法使用，请检查浏览器是否启用了代理；') + lineBreak;
                    rsDataItem.state = false;
                }
            }

            if (rsDataItem.type !== 0 && !resConfig.config.ecIsExist) { // 是否安装客户端
                rsDataItem.state = false;
                rsDataItem.msg += circleSymbol + tr('您尚未安装EC客户端；') + lineBreak;
            }
            if (rsDataItem.msg !== '') {
                rsDataItem.msg = tr('该资源无法使用，原因如下：') + lineBreak + rsDataItem.msg;
            }
            if (rsDataItem.isAddrRange && rsDataItem.state) { // IP段资源无法点击，但不显示禁用状态，资源禁用时不显示
                rsDataItem.msg += tr('该资源为IP段资源，不支持直接点击访问；') + lineBreak;//该资源为IP段资源，不支持直接点击访问
            }
            if (!rsDataItem.state && !resConfig.rc.config.exitDisabledRs) {
                resConfig.rc.config.exitDisabledRs = true; //标记存在被禁用的资源
            }
            return rsDataItem;
        },
        openRemoteApp: function (rsItem) {
            SFAPI.doConfigEC({
                key: 'C_RMTAPP',
                params: {
                    id: rsItem.id,
                    appPath: rsItem.app_path
                },
                success: function (data) {
                    log.info(LOGTAG, ' openRemoteApp: Open remoteAPP success');
                },
                error: function (res) {
                    log.error(LOGTAG, 'openRemoteApp: Failed Open remoteAPP', ' error is %s ', res.msg);
                }
            });
        },
        handWebHref: function (id, isDefault) {
            var webData = _getRcById(id), // 得到资源
                _this = this,
                hostFrom;

            if (is.empty(webData)) {
                log.debug(LOGTAG, 'Resource: Data not exsist! id : ' + id);
                return;
            }

            hostFrom = webData.ips[0].host_from;

            _this.sf_handle_websvc(0, hostFrom, id, isDefault);
        },
        handleResource: function (id, index, isDefault) {

            var rc = _getRcById(id),
                _this = this;

            if (rc.svc === rcService.REMOTE_APP || (is.Mac && rc.svc === rcService.TERMINAL_SERVICE)) { // 远程应用
                _this.openRemoteApp(rc);
            } else if (rc.type === rcType.WEB) { // handWebHref与sso
                _this.handWebHref(rc.id, isDefault);
            } else if (rc.type === rcType.APP) { // tcp与远程应用
                _this.handleAppHref(rc.id, index, isDefault); // SinforHandleAppHref
            } else if (rc.type === rcType.IP) { // L3VPN
                _this.handleIpHref(rc.id, index, isDefault); // SinforHandleIpHref
            } else {
                SFLOG.warn(LOGTAG, 'Handle resource: Unknow Resource type', 'May be data error');
            }
        },
        handleIpHref: function (id, index, isDefault) {
            var rc = _getRcById(id);

            SFLOG.assert(!is.empty(rc), LOGTAG + 'handleIpHref get rc result is empty');

            if (is.set(index)) {
                index = Number(index);
            }

            index = is.empty(index) ? 0 : index;

            //是否是超链接
            if (!_isHref(rc, index)) {
                SFLOG.info(LOGTAG, 'handleIpHref want to opened is not a href. Not go to anywhere.');
                return;
            }

            //打开L3vpn资源
            _openL3vpnResource(id, index, isDefault);
        },
        handleAppHref: function (id, index, isDefault) {
            var rc = _getRcById(id);
            SFLOG.assert(!is.empty(rc), LOGTAG + 'handleIpHref get rc result is empty');

            if (is.set(index)) {
                index = Number(index);
            }

            index = is.empty(index) ? 0 : index;

            //是否是超链接
            if (!_isHref(rc, index)) {
                SFLOG.info(LOGTAG, 'handleAppHref want to opened is not a href. Not go to anywhere.');
                return;
            }
            _openTcpResource(id, index, isDefault);
        },

        // 处理单点登录
        handleSso: function (rctype, id, isDefault, index, ssodata) {
            var ret = 0,
                rc,
                rcid,
                service,
                flag = 0,
                _this = this,
                csBit = 3; // 位移3位
            isDefault = (!is.set(isDefault)) ? false : isDefault;
            index = (!is.set(index)) ? 0 : index;

            /*if (!resConfig.sso.obj) {
                log.debug(LOGTAG, 'handleSso: SSO: SSO object is null!');
                return false;
            }*/
            rc = _getRcById(id);
            if (is.empty(rc)) {
                log.debug(LOGTAG, 'handleSso RC: Result of rc is null!');
                return false;
            }
            rc = _buildHostPro(rc, index);
            rcid = rc.id;
            if (SFConfig.isExistEC) { // 当前是否为EC
                // 调用EC接口

            }


            SFLOG.assert(is.number(rctype), 'rctype is not number type');
            if (rcType.WEB === rctype) {
                if (rc.svc === rcService.FTP) {
                    _handleFtp(rc.enable_disguise, rc.host_from, rc.port_from, isDefault, rcid);
                } else if (rc.svc === rcService.MAIL) {
                    _handleMail(rc.enable_disguise, rc.host_from, rc.port_to, rc.app_path, isDefault, rcid);
                } else {
                    _this.sf_handle_websvc(index, rc.host_from, id, isDefault);
                }
            } else {
                service = rc.svc;
                if (service === rcService.HTTP || service === rcService.HTTPS) {//bs
                    // !!@@ 单点登录逻辑改动

                    SFCommon.openWindow({url: _getSsoParams(rc, rc.host_from)});

                    // 单点登录!!@@
                } else { // cs模式
                    if ((rc.attr & (1 << csBit)) !== 0) {
                        flag = 1;
                    }

                    // todo 确认: cs模式的, 不应该跳转中转页面, 页面无法关闭
                    // ret = resConfig.sso.obj.SetSSOConfigUB(resConfig.sso.data[rcid].data, 0, rc.app_path, '', rctype, flag);

                    API.startSsoProgram({
                        args: [resConfig.sso.data[rcid].data, 0, rc.app_path, '', rctype, flag],
                        success: function () {
                            log.debug(LOGTAG, 'Open resource for cs sso success');
                        }
                    });

                }
            }

            //return ret != -1;
        },

        /**
         * 处理web服务
         * @param {int} index
         * @param {string} url
         * @param {init} rcid
         * @param {bool} isDefault
         * @param {object} rsItem
         */
        sf_handle_websvc: function (index, url, rcid, isDefault) {
            var realvpnaddr,
                rcInfo,
                tmpurl,
                realurl,
                isDefaultPage = 0,
                ret,
                mark,
                _this = this,
                locationPort = window.location.port;
            // eslint-disable-next-line
            if (arguments.length === 2) {//说明是老模板的全网资源那里调用的
                rcid = url;
                url = index;
                index = 0;
            }

            realvpnaddr = window.location.hostname;

            //如果是默认资源且为mail或FTP，则加入相应处理
            if (resConfig.rc.defaults.rcId === rcid && resConfig.rc.defaults.rcId > 0) {
                rcInfo = _getRcById(resConfig.rc.defaults.rcId);
                if (rcInfo === null) {
                    return;
                }
                rcInfo = _buildHostPro(rcInfo, index);
                if (rcInfo.svc === rcService.FTP) {
                    _handleFtp(rcInfo.enable_disguise, rcInfo.host_from, rcInfo.port_from, isDefault);
                    return;
                } else if (rcInfo.svc === rcService.MAIL) {
                    _handleMail(rcInfo.enable_disguise, rcInfo.host_from, rcInfo.port_to, rcInfo.app_path, isDefault);
                    return;
                } else if (rcInfo.svc === rcService.FILESHARE) {
                    _handleFileShare(rcid, isDefault);
                    return;
                }
            }

            if (resConfig.rc.easyLinkHash[rcid]) {
                _handEasyLinkHref(rcid, isDefault);
                return;
            }

            tmpurl = String(url);
            if (tmpurl === '' || tmpurl === '?') {
                return;
            }
            tmpurl = (tmpurl.replace(/^\s+/g, '')).replace(/\s+$/g, '').replace(/\\/g, '/');
            if (!tmpurl.match(/^(http:\/\/)|(https:\/\/)\s*/i)) {	//没有http 或 https 开头
                tmpurl = 'http://' + tmpurl;
            }

            if (!tmpurl.match(/^https?:\/\/.+\//i)) {
                if (tmpurl.indexOf('?') >= 0) {
                    tmpurl = tmpurl.replace(/\?/, '\/?');
                } else if (tmpurl.indexOf('#') >= 0) {
                    tmpurl = tmpurl.replace(/#/, '\/#');
                } else {
                    tmpurl = tmpurl + '/';
                }
            }
            rcInfo = _getRcById(rcid);
            if (rcInfo && rcInfo.enable_disguise) {
                tmpurl = _sfTransUrl(tmpurl, true);
            } else {
                tmpurl = _sfTransUrl(tmpurl);
            }
            if (tmpurl === '') {
                return;
            }

            realurl = '';

            /*lsc add for websso*/
            // eslint-disable-next-line
            if (rcid > 0 || rcid < -1000) { // !!@@ 单点登录逻辑改动
                if (!((resConfig.rc.defaults.rcId > 0) && isDefault)) {
                    realurl = tmpurl;
                } else {
                    realurl = 'https://' + realvpnaddr + (!locationPort ? '' : ':' + locationPort) +
                        '/' + session.getTWFID() + tmpurl;
                }
                ret = _sfHandleWebsso(rcid, realurl, isDefault);
                if (ret === 1) {
                    return;
                } else if (ret !== 0) {
                    tmpurl = ret;
                }
            }

            realurl = 'https://' + realvpnaddr + (!locationPort ? '' : ':' + locationPort) + tmpurl;

            if ((resConfig.rc.defaults.rcId > 0) && isDefault) {

                realurl = 'https://' + realvpnaddr + (!locationPort ? '' : ':' + locationPort) + '/' + session.getTWFID() + tmpurl;
                if (!(resConfig.sso.data.length > 0 && _isCanUseSSO(resConfig.sso.data[rcid]))) {

                    // no sso & default rc
                    realurl = 'https://' + realvpnaddr + (!locationPort ? '' : ':' + locationPort) + '/' + session.getTWFID() + tmpurl;

                    _openWebResource(rcInfo, realurl, false, false, isDefault);
                } else {

                    // sso
                    _openWebResource(rcInfo, realurl, false, true, isDefault);
                    return;

                    /*
                    // todo defaultpage not use ;
                    realurl = 'https://' + realvpnaddr + (!locationPort ? '' : ':' + locationPort) +
                        '/com/defaultpage.htm?redirect=1&TWFID=' + session.getTWFID() + '&' + tmpurl;
                    mark = '';
                    if (is.set(resConfig.sso.data[rcid].mark)) {
                        mark = resConfig.sso.data[rcid].mark;
                    }
                    resConfig.sso.obj.CreateIEWithHandle(realurl, mark);
                    */
                }
            } else {

                // not default rc
                try {
                    if ((resConfig.sso.data.length > 0) && _isCanUseSSO(resConfig.sso.data[rcid])) {
                        if (tmpurl.indexOf('?') !== -1) {
                            realurl = tmpurl + '&' + resConfig.sso.obj.GetSSOLoginString;
                        } else {
                            mark = '?';
                            if (is.set(resConfig.sso.data[rcid].mark)) {
                                mark = resConfig.sso.data[rcid].mark;
                            }
                            realurl = tmpurl + mark + resConfig.sso.obj.GetSSOLoginString;
                        }
                    }

                    _openWebResource(rcInfo, realurl, false, false, isDefault);
                } catch (e) {
                    SFLOG.info(LOGTAG, 'sf_handle_websvc() error' + e.message); // todo
                }
            }
        },

        // 默认资源跳转
        // return bool 是否当前页面跳转
        goDefaultPage: function () { // 处理默认服务页面
            var defaultRcURL,
                rcInfo = _getRcById(resConfig.rc.defaults.rcId), //如果没有设置默认资源,resConfig.rc.defaults.rcId的值为0,表示是ip全网资源;
                regExp = /^(((https?)|(ftp)):\/\/)?([^:\/\?]+)(:(\d+))?([^:\?]*)?(\?.*)?$/ig,
                def = resConfig.rc.defaults, // 处理默认资源的URL效验，没有协议则缺省为HTTP。
                SSOData,
                hostFrom,
                protocol,
                host,
                port,
                subforder,
                getParam,
                _this = this,
                id = Number(resConfig.rc.defaults.rcId);

            log.info(LOGTAG, 'exist default id ,rcid is ' + id);

            if (id <= 0) {
                log.debug(LOGTAG, 'exist default id,default rcid <=0 , not continue ');
                return false;
            }

            //处理不支持控件的浏览器下面判断IP、APP资源为默认资源的情况
            if (!rcInfo) {
                log.debug(LOGTAG, 'exist default id,all rc not exist default id , not continue ');
                return false;
            }
            SFLOG.assert(is.number(def.rc.type), 'def.rc.type is not number');

            if (!def.rcId) {
                log.debug(LOGTAG, 'not exist default id in info of default rc, not continue ');
                return false;
            }

            // 依赖EC的资源, 无EC不跳转
            if (!G.isExistEC && (def.rc.type === rcType.APP || def.rc.type === rcType.IP)) {
                log.debug(LOGTAG, 'default rc is TCP or L3VPN, but ec is not exist, not continue');
                return false;
            }

            // 如果当前跳转的资源是L3VPN，并且是第二个登录的客户端，则不支持自动跳转。
            if ((def.rc.type === rcType.IP) && !isFirstL3vpn) {
                log.debug(LOGTAG, 'Exist default id, But not first L3vpn service, not continue ');
                return false;
            }

            rcInfo = _buildHostPro(rcInfo);
            if (!regExp.test(rcInfo.host_from)) {
                log.debug(LOGTAG, 'host_from of rcinfo,not match rules , not continue ');
                return false;
            }


            // todo 返回当前页面跳转
            _this.handleResource(id, 0, true);

            // ec 需要关闭当前窗口;
            if (!com.isWeb()) {
                log.debug(LOGTAG, 'Hide EC window after open Default resource.');

                // 客户端下, 打开默认资源后, 自动关闭当前ec窗口.
                SF.windowMgr.hide(G.CONTAINER_WINDOW_ID.CURRENT_WINDOW);
            }

            return true;

            // todo remove code after

            hostFrom = rcInfo.host_from;
            hostFrom.replace(regExp, function () {
                var args = arguments;
                protocol = args[2] || '';
                host = args[5] || '';
                port = args[7] || '';
                subforder = args[8] || '';
                getParam = args[9] || '';
            });

            protocol = !protocol ? (rcInfo.svc === rcService.HTTP ||
            rcInfo.svc === rcService.HTTPS ? rcInfo.svc : 'http') : protocol; // 请求协议
            port = (!port) ? (rcInfo.port_from.toString() !== '0' ? rcInfo.port_from : '80') : port;//请求端口

            defaultRcURL = protocol + '://' + host + ':' + port + subforder + getParam;

            SFLOG.assert(is.number(rcInfo.type), 'goDefaultPage() rcInfo.type is not number');

            // 如果是自动填表，且为IE
            if (resConfig.sso.data.length > 0 && _isCanUseSSO(resConfig.sso.data[rcInfo.id])) { // ie才会进这个逻辑
                SSOData = resConfig.sso.data[rcInfo.id];
                SF.res.handleSso(rcInfo.type, rcInfo.id, true, 0, SSOData);
            } else { // 如果自动构建参数会sf_handle_websvc里面处理完URL后处理
                if (rcInfo.type === rcType.WEB) {
                    SF.res.sf_handle_websvc(0, rcInfo.host_from, rcInfo.id, true);
                    return true;
                } 
                if (!(is.fn(_sfHandleWebsso) && _sfHandleWebsso(rcInfo.id, defaultRcURL, 1))) {
                    SFCommon.openWindow({url: defaultRcURL, target: false});
                    return true;
                }
                
            }
            return true;
        },
        swicthShowRcList: function () {
            var vpnInfo, prefData, isShowRc;

            // ec 才需要根据配置决定是否关闭当前窗口
            if (com.isWeb()) {
                return;
            }

            vpnInfo = SF.vpnInfo.createInstance();
            prefData = vpnInfo.get();

            // 未保存过配置,
            if (is.set(prefData.showRc)) {
                isShowRc = prefData.showRc;
                log.debug(LOGTAG, 'Get showRc from pref-settings, showRc:' + isShowRc);
            } else {
                isShowRc = resConfig.config.showRc;
            }

            if (isShowRc) {
                log.debug(LOGTAG, 'Not hide EC window, Because showRc of pref-settings or server is false');
            } else {
                log.debug(LOGTAG, 'Hide EC window, Because showRc of pref-settings or server is true');
                SF.windowMgr.hide(G.CONTAINER_WINDOW_ID.CURRENT_WINDOW);
            }
        }
    };


    /*
     在enablesso方法里有调到
     */
    function _configSso () {
        var rc,
            key,
            oneSecond = 1000,
            flag,
            useUb,
            i,
            useUbBit = 3;
        if (!resConfig.sso.obj) {
            return false;
        }
        function _isBS (service) {
            service = service.toUpperCase();
            return service === 'HTTP' || service === 'HTTPS';
        }

        resConfig.sso.obj.SetLocaleInfo(resConfig.info.msgSSOClientPrj);
        resConfig.sso.obj.Login();
        resConfig.sso.obj.setUserInfo(resConfig.sso.info.username, resConfig.sso.raw.pwd);
        resConfig.sso.obj.SetTimeout(resConfig.sso.config.ssotimeout * oneSecond);

        for (i = 0; i < resConfig.sso.data.length; i++) {
            rc = _getRcById(resConfig.sso.data[i]);
            if (!is.empty(rc) && (_isBS(rc.svc) || (rc.type === rcType.WEB))) {
                flag = 0;

                useUb = rc.attr;
                if ((useUb & 1 << useUbBit) !== 0) {
                    flag = 1;
                }
                resConfig.sso.obj.SetBsSSOConfig(resConfig.sso.data[key].data, resConfig.info.pageHref, rc.type, flag);
            }
        }

        resConfig.sso.obj.SetBsConfigEnd();
        resConfig.sso.obj.SetHandleInfo();
        return true;
    }

    function _disableSso () {
        var web2ClientName,
            ssoClient;
        try {
            if (!!resConfig.sso.obj) {
                resConfig.sso.obj.StopSSO();
            } else {
                web2ClientName = 'SSOClientPrj.Web2Client' + resConfig.config.sslvpnNameFix + '.1';

                // todo

                // eslint-disable-next-line
                ssoClient = CreateObject(web2ClientName);
                ssoClient.StopSSO();
            }
            resConfig.sso.config.enable = 0;
        } catch (e) {
            SFLOG.warn(LOGTAG, 'disabledSso error() ',  e.message);
        }
    }

    /**
     * 初始化单点登录数据
     * @returns {boolean}
     */
    function _initSsoData () {
        var info = ['', '', ''],
            indexTmp,
            psw,
            idx,
            ssoData,
            ssoArr,
            certInfoLen = 3, // 用于证书信息长度判断
            certInfoBaseLen = 2;

        log.info(LOGTAG, 'step :initSsoData  ');

        if (!resConfig.rc.raw.Resource.Sso) {
            log.info(LOGTAG, ' sso data is empty ');
            return false;
        }
        resConfig.sso.raw = resConfig.rc.raw.Resource.Sso;
        resConfig.sso.config.enable = _getRcValue('Sso.enable', 0);
        resConfig.sso.config.show_ssoconf = _getRcValue('Sso.show_ssoconf', 0);
        resConfig.sso.config.show_perssoconf = _getRcValue('Sso.show_perssoconf', 0);
        resConfig.sso.config.ssotimeout = _getRcValue('Sso.ssotimeout', 0);


        function _getSsoInfo (value) {
            var ret = '';
            if (is.set(value) && value !== false) {
                ret = encodeURIComponent(value);
            }
            return ret;
        }

        //外置CA用户名重名时去除单点登录用户名中的证书信息
        resConfig.sso.info.username = resConfig.info.loginName.replace(/<.+>$/, '');


        if (resConfig.sso.info.cert) {

            //G.sso.info.cert在initGlobal()中已经初始化
            info = resConfig.sso.info.cert.split('<>');
            if (info.length >= certInfoLen) {
                indexTmp = resConfig.sso.info.cert.indexOf(info[1]) + info[1].length + certInfoBaseLen;
                info[2] = resConfig.sso.info.cert.substring(indexTmp);
            }
        }

        resConfig.sso.info.phone_num = _getSsoInfo(resConfig.sso.raw.phone_num);
        resConfig.sso.info.group_name = _getSsoInfo(resConfig.sso.raw.groupname);
        resConfig.sso.info.scert_thum = _getSsoInfo(info[0]);
        resConfig.sso.info.scert_sn = _getSsoInfo(info[1]);
        resConfig.sso.info.scert_info = _getSsoInfo(info[2]);

        //resConfig.sso.info.dkey_id = getSsoInfo(getCookie("DkeySerialNumber"));

        ssoData = _getRcValue('Sso.data', '');
        if (ssoData) {
            ssoArr = ssoData.split(';');
            ssoArr.forEach(function (val) {
                var kv = val.split(':');
                if (!resConfig.sso.data) {
                    resConfig.sso.data = {};
                }
                resConfig.sso.data[kv[0]] = {data: kv[1]};
                if (is.set(kv[2]) && kv[2]) {
                    resConfig.sso.data[kv[0]].mark = kv[2];
                }
            });
        }

        //websso
        resConfig.websso = _buildArray('Resource.WebSsoInfos.webssoinfo', resConfig.rc.raw);
        resConfig.websso.forEach(function (ssoinfo) {
            ssoinfo.args = _buildArray('args', ssoinfo);
            ssoinfo.args.forEach(function (arg) {
                arg.value = _htmlDecode(arg.value);
            });
        });
        log.info(LOGTAG, '_initSsoData() success');
    }

    // 是否可以用sso
    function _isCanUseSSO (Id) {
        return (resConfig.sso.config.enable &&
            resConfig.sso.data.length > 0 &&
            Id //单点登录配置有该资源
        );
    }

    /**
     * 初始化单点登录组件
     */
    /*
    function _initSso () {
        _dealSSO();
    }
    function _dealSSO () {
        log.info(LOGTAG, 'step : dealSSO  ');
        if (resConfig.sso.config.enable && resConfig.sso.data.length > 0) {
            if (!_enableSso()) {
                log.info(LOGTAG, 'initSso:component: disableSso ');
                _disableSso();
            } else {
                log.info(LOGTAG, 'initsso:component: success ');
            }
        }
    }
    */

    //braveboy，这里要处理切换到单点登录负载均衡资源的时候，安装CSCM控件的情况。
    function _enableSso (isBalance) {

        // todo 对象是否存在

        // eslint-disable-next-line
        /*
        if (!(_makeSsoOK() && _configSso())) {
            return false;
        }*/
        return true;
    }

    /*
    function _makeSsoOK () {
        var web2ClientName;
        try {
            web2ClientName = 'SSOClientPrj.Web2Client' + resConfig.config.sslvpnNameFix + '.1';
            // eslint-disable-next-line
            resConfig.sso.obj = new ActiveXObject(web2ClientName);
        } catch (e) {
            resConfig.sso.obj = null;
            return false;
        }
        return true;
    }
    */

    /**
     * 退出sso
     * @param {number} id
     * @return {boolean} 是否存在
     */
    /*
    function _existSso (id) {
        var isexist = typeof(id);
        return 'undefined' !== isexist && 'unknown' !== isexist;
    }
    */

    // 打开TCP资源
    function _openTcpResource (id, index, isDefault) {
        var rc = _getRcById(id),
            service = rc.svc,
            hrefData,
            host,
            ret,
            paramsData,
            SSOData;

        //extend为复制属性方法
        hrefData = _extend({}, rc.ips[index]);
        host = _handleAppHrefHelper(_extend(hrefData, rc.ports[index]), service);

        // !!@@ 单点登录逻辑改动
        ret = _sfHandleWebsso(id, host, isDefault);
        SFLOG.assert(is.number(ret), '_openTcpResource() ret is not number');
        if (ret === 1) {
            return;
        } else if (ret !== 0) {
            host = ret;
        }

        if (resConfig.sso.data.length > 0 && _isCanUseSSO(resConfig.sso.data[rc.id])) {//单点登录配置有该资源
            SSOData = resConfig.sso.data[rc.id];
            SF.res.handleSso(1, rc.id, isDefault, index, SSOData);
            return;
        }
        if (service === rcService.HTTP || service === rcService.HTTPS) {
            SFCommon.openWindow({url: host, target: !isDefault});
            return;
        }

        openLocalApp(rc, host);
    }

    // 打开本地程序
    function openLocalApp (rc, host) {
        var address, paramsData, service;

        service = rc.svc;
        if (!(service === rcService.FTP || service === rcService.FILESHARE)) {
            address = rc.app_path;
        } else if (service === rcService.FTP && rc.app_path !== '') {
            address = Base64.encode(host + '|' + Base64.decode(rc.app_path));
        } else {
            address = Base64.encode(host);
        }

        paramsData = Base64.encode(format('{0}|{1}|{2}', rc.id, rc.name, address));

        SFAPI.doConfigEC({
            key: 'C_LOCALAPP',
            params: {
                data: paramsData
            },
            success: function (data) {
                log.info(LOGTAG, '_openTcpResource() open resource by ECAgent success');
            },
            error: function (err) {
                log.error(LOGTAG, 'Failed to open resource', ' error is %s ', err.msg);
            }
        });
    }



    /**
     * 生成app链接的辅助函数
     * @param {object} rc
     * @param {string} service
     * @returns {string}
     */
    function _handleAppHrefHelper (rc, service) {
        var host = _handleIpHrefHelper(rc, service);
        return host;
    }

    /**
     * 复制对象属性函数
     * @param {object} obj1 将被添加属性的对象
     * @param {object} obj2 被复制属性的对象
     * @return {object} obj1 新对象
     */
    function _extend (obj1, obj2) {
        var o;
        for (o in obj2) {
            if (obj2.hasOwnProperty(o)) {
                obj1[o] = obj2[o];
            }
        }
        return obj1;
    }

    // 打开L3vpn资源
    function _openL3vpnResource (id, index, isDefault) {
        var rc = _getRcById(id),
            service = rc.svc,
            hrefData = _extend({}, rc.ips[index]), // extend为复制属性方法
            host = _handleIpHrefHelper(_extend(hrefData, rc.ports[index]), service),
            ret;

        if (is.fn(_sfHandleWebsso)) {
            ret = _sfHandleWebsso(rc.id, host, isDefault);
            SFLOG.assert(is.number(ret), '_openL3vpnResource() ret is not number');
            if (ret === 1) {
                return;
            } else if (ret !== 0) {
                host = ret;
            }
        }

        if (resConfig.sso.data.length > 0 && _isCanUseSSO(resConfig.sso.data[rc.id])) {

            //单点登录配置有该资源
            SF.res.handleSso(1, rc.id, isDefault, index, resConfig.sso.data[rc.id]);
            return;
        }
        if (service === rcService.HTTP || service === rcService.HTTPS) {
            SFCommon.openWindow({url: host, target: !isDefault});
            return;
        }
        openLocalApp(rc, host);
    }

    function _sfGetMyword (str) {
        var e = /\<\%([a-z_]{5,20})\%\>/i,
            r = null,
            s = '',
            start = 0,
            end = 0,
            rstr = '',
            myword = '',
            key;

        str = decodeURIComponent(str);

        // eslint-disable-next-line
        while (r = e.exec(str)) {

            end = r.index;
            myword += str.substring(start, end);
            key = r[1];

            if (typeof(resConfig.sso.info[key]) !== 'undefined') {
                myword += decodeURIComponent(resConfig.sso.info[key]);
            } else {
                myword += r[0];
            }
            str = str.substr(end + r[0].length);
        }
        myword += str;
        return myword;
    }

    /**
     * 生成app链接的辅助函数
     * @param {object} rc
     * @param {string} service
     * @returns {string} host
     */
    function _handleIpHrefHelper (rc, service) {
        var port = '',
            host = '',
            index1,
            index2;

        SFLOG.assert(is.string(rc.port_from), 'rc.port_from is not string type');

        if (service === rcService.HTTP) {
            port = rc.port_from === '80' ? '' : ':' + rc.port_from;
            service = 'http://';
        } else if (service === rcService.HTTPS) {
            port = rc.port_from === '443' ? '' : ':' + rc.port_from;
            service = 'https://';
        } else if (service === rcService.FILESHARE) {
            service = '\\\\';
        } else if (service === rcService.FTP) {
            port = rc.port_from === '21' ? '' : ':' + rc.port_from;
            service = 'ftp://';
        }
        index1 = rc.host_from.indexOf('://') || rc.host_from.indexOf('\\\\');
        index2 = rc.host_from.indexOf(':', index1 + 1);

        //if(index2 > 0 || rc['port_to'] != rc['port_from']){//will r 这里好象有问题 现在用if(index2 > 0){替代
        if (index2 > 0) {
            host = rc.host_from;
        } else {
            host = rc.host_from + port;
        }
        host = (index1 === -1) ? (service + host) : host;
        return host;
    }

    /**
     * 构建主机地址相关属性
     * @param {object} rc
     * @param {number} index
     * @returns {object}
     */
    function _buildHostPro (rc, index) {
        index = is.empty(index) ? 0 : new Number(index);
        rc.host_from = rc.ips[index].host_from;
        rc.host_to = rc.ips[index].host_to;
        rc.port_from = rc.ports[index].port_from;
        rc.port_to = rc.ports[index].port_to;
        return rc;
    }

    /**
     * 是否超链接
     * @param {Array} rcArray
     * @param {Number} index 第几个连接
     * @returns {boolean}
     */
    function _isHref (rcArray, index) {
        var service;
        rcArray = _buildHostPro(rcArray, index);
        service = rcArray.svc;
        SFLOG.assert(is.number(rcArray.id), 'rcArray.id is not number type');
        if (rcArray.id === 0) {
            return false;
        }

        SFLOG.assert(is.number(rcArray.type), 'rcArray.type is not number type');
        if ((rcArray.type === rcType.IP) && resConfig.info.isUseIeProxy) {//ie代理也不行
            return false;
        }

        SFLOG.assert(is.number(rcArray.proto), 'rcArray.proto is not number type');

        if ((service === rcService.DNS) || (rcArray.proto === rcProto.ICMP && rcArray.type === rcType.IP)) {

            //dns,icmp 是不可以点的
            return false;
        }

        SFLOG.assert(is.string(rcArray.host_to), 'rcArray.host_to is not string type, type: ' + typeof(rcArray.host_to));

        //ip地址段 ，端口段，都不能访问
        if ((rcArray.port_from !== rcArray.port_to) ||
            (rcArray.host_from !== rcArray.host_to && rcArray.host_to)) {//url地址不满足
            if (rcArray.type === rcType.WEB) {//url不合法的web资源
                return false;
            }

            if (service === rcService.HTTP || service === rcService.HTTPS) {//url不合法的app/ip资源
                return false;
            }


        }
        if (!is.Win()) {

            //非win平台上web资源url合法的可以点击，ip,tcp资源凡是win上通过控件打开资源都不可以点击(除http,https都是控件打开的)
            // mac 上可以使用: 远程应用 | terminal
            if (is.Mac && (service === rcService.REMOTE_APP  || service === rcService.TERMINAL_SERVICE)) {
                return true;
            }
            if ((rcType.WEB === rcArray.type) || (service === rcService.HTTP) || (service === rcService.HTTPS)) {
                return true;
            }
            return false;
        }

        //以下为url合法
        if (
            service === rcService.FTP ||
            service === rcService.FILESHARE ||
            service === rcService.HTTP ||
            service === rcService.HTTPS ||
            rcArray.type === rcType.WEB
        ) {

            //url合法的app/ip资源,或者web资源
            return true;
        }

        //以下为非web类型,且不为ftp,fileshare,http,https
        if (is.empty(rcArray.app_path.trim())) { // 非web资源又没有app_path
            //return false;
            return true;
        }
        return true;
    }

    /**
     * EasyLink资源链接
     * @param {number} rcid
     * @param {array} isDefault 是否默认资源，false（含空、0等值）为非
     */

    function _handEasyLinkHref (rcid, isDefault) {
        var modes = {
                port: '0',
                domain: '1',
                subdomain: '2'
            },
            regExp = /^(((https?)|(ftp)):\/\/)?([^:\/]+)(:(\d+))?([^:]*)?$/i,
            rc = _getRcById(rcid);

        if (is.empty(rc)) {
            log.error(LOGTAG, 'handEasyLinkHref: handle error  ', ' rc is null');
            return;
        }

        // eslint-disable-next-line
        var rcInfo = _buildHostPro(rc, 0),
            hostFrom = rcInfo.host_from,
            subforder = hostFrom.replace(regExp, '$8').replace(/&$/, '').replace(/\?$/, ''),
            easyLinkRc = resConfig.rc.easyLinkHash[rcid],
            easyLinkURL = easyLinkRc.MapAddr,
            hash = '',
            redirect = '',
            twfidParam = '',
            mode = easyLinkRc.Mode,
            httpPort = resConfig.config.httpPort,
            port = window.location.port,
            url,
            httpsLength = 6,
            noWebssoIDEnd = 0,
            noWebssoIDStart = -1000,
            ret,
            isDefaultPage,
            realurl,
            isEnableHttp = !!(is.set(resConfig.config.enableWebHttp) && Number(resConfig.config.enableWebHttp));

        SFLOG.assert(is.string(mode), 'mode is not string type');


        if (!is.empty(subforder)) {
            subforder = subforder.replace(/#.*$/i, function ($0) {
                hash = $0;
                return '';
            });
        } else {
            easyLinkURL = easyLinkURL.replace(/#.*$/i, function ($0) {
                hash = $0;
                return '';
            });
        }

        easyLinkURL = easyLinkURL.trim().replace(/\/$/, '');


        if (isEnableHttp && httpPort > 0 &&
            (mode === modes.domain || mode === modes.subdomain)) {

            // 开启了http透明代理, 域名,泛域名资源: https 替换为http协议
            if (easyLinkURL.substr(0, httpsLength).toLowerCase() === 'https:') {
                easyLinkURL = 'http:' + easyLinkURL.substr(httpsLength);
                port = httpPort;
            }
        }

        if (mode !== modes.port) {
            if (port) {
                easyLinkURL = easyLinkURL + ':' + port;
            }
            redirect = subforder.indexOf('?') !== -1 ? '&sangfor_redirect=1' : '?sangfor_redirect=1';

            // eslint-disable-next-line
            twfidParam = '&sangfor_sessid=' + Base64.encode(session.getTWFID());
        }


        // todo format move to common file
        // eslint-disable-next-line
        url = format('{0}{1}{2}{3}{4}', easyLinkURL, subforder, redirect, twfidParam, hash);

        /*lsc add for websso*/
        if ((rcid > noWebssoIDEnd || rcid < noWebssoIDStart) && (is.fn(_sfHandleWebsso))) {
            ret = _sfHandleWebsso(rcid, url, isDefault);
            SFLOG.assert(is.number(ret), 'ret of _sfHandleWebsso() is not number type');

            if (ret === 1) {
                return;
            }
        }
        if ((resConfig.rc.defaults.rcId > 0) && isDefault) {

            // setCookie('g_LoginPage', '0');
            // eslint-disable-next-line
            var realvpnaddr = window.location.hostname;

            if (resConfig.sso.data.length > 0 && _isCanUseSSO(resConfig.sso.data[rcid])) {

                // sso

                _openWebResource(rcInfo, url, true, true, isDefault);

                /*// todo defaultpage.htm
                realurl = 'https://' + realvpnaddr + (!window.location.port ? '' : ':' + window.location.port) +
                    '/com/defaultpage.htm?redirect=1&TWFID=' + session.getTWFID() + '&' + url;
                resConfig.sso.obj.CreateIEWithHandle(realurl, '');*/

            } else {

                // no sso
                SFCommon.openWindow({url: url, target: !isDefault});
            }
        } else {

            if ((resConfig.sso.data.length > 0) && _isCanUseSSO(resConfig.sso.data[rcid])) {
                _openWebResource(rcInfo, url, true, true, isDefault);
            } else {
                _openWebResource(rcInfo, url, true, false, isDefault);
            }

            // SFCommon.openWindow({url: url});

        }
    }

    // 转换url
    function _sfTransUrl (theurl, hideHost) {
        var tmpstr = '/web/',
            filter = '1',
            tmpurl = new String(theurl),
            exp = /(https?)(:\/\/)(.*)$/i,
            result,
            host;

        tmpurl = tmpurl.replace(/[\?#].*$/, '');//remove the querystring;
        if (tmpurl.match(/\.css$/i)) {
            filter = '2';
        } else if (tmpurl.match(/\.js$/i)) {
            filter = '3';
        } else if (tmpurl.match(/\.vbs$/i)) {
            filter = '4';
        }

        result = theurl.match(exp);
        if (!result) {
            return '';
        }
        host = result[3];
        if (hideHost) {
            tmpstr = STR_FAKE_URL + tmpstr;
        }
        return tmpstr + filter + '/' + result[1] + '/0/' + host;
    }

    function _sfHandleWebsso (rcId, rcHref, isDefault) {
        var flag = 0,
            row,
            i,
            rcInfo = _getRcById(rcId),
            noWebssoIDEnd = 0,
            noWebssoIDStart = -1000;

        if (is.empty(resConfig.websso) && (rcId < noWebssoIDEnd && rcId > noWebssoIDStart)) {
            return 0;
        }

        for (i = 0; i < resConfig.websso.length; i++) {
            if (rcId === resConfig.websso[i].rcid) {
                flag = i + 1;
                break;
            }
        }
        if (rcHref === '' || flag === 0) {//判断有没有这个websso资源。
            return 0;
        }

        row = resConfig.websso[flag - 1];//获取WEBSSO信息。
        if (is.empty(row.args)) {
            return 0;
        }

        SFCommon.openWindow({url: _getSsoParams(rcInfo, rcHref), target: !isDefault});
        return 1;
    }

    // 处理FileShare资源
    function _handleFileShare (rcid, isDefault) {
        var rcURL = '/por/webfs.csp?rcid=' + rcid + '&rnd=' + Math.random(),
            rcSSOInfo = null,
            fileSsoInfo,
            i;
        for (i = resConfig.websso.length; i--;) {
            if (rcid === resConfig.websso[i].rcid) {
                rcSSOInfo = resConfig.websso[i];
                break;
            }
        }
        if (rcSSOInfo) {//如果有下发了该资源的参数名称，说明可以取到其对应的值。
            fileSsoInfo = {
                username: '',
                password: '',
                domain: ''
            };
            for (i = rcSSOInfo.args.length; i--;) {

                // todo sf_get_value is not defined, make sure to del it.
                fileSsoInfo[rcSSOInfo.args[i].name.toLowerCase()] = sf_get_value(_sfGetMyword(rcSSOInfo.args[i].value));
            }

            // todo temp
            // setCookie('WEBFS_SSO_' + rcid, JSON.stringify(fileSsoInfo));
            rcURL += '&sso=1';
        }

        SFCommon.openWindow({url: rcURL, target: !isDefault});
    }

    // 通过资源组id获取资源组信息
    function _getRcGrpById (id) {
        var groups = resConfig.rc.groupsHash[id];
        return is.empty(groups) ? null : groups;
    }

    // 通过资源id获取资源信息
    function _getRcById (id) {
        var rcs = resConfig.rc.rcsHash[id];
        return is.empty(rcs) ? null : rcs;
    }

    // 处理ftp资源
    function _handleFtp (enableDisguise, host, port, isDefault, rcid) {
        var tmpurl,
            realurl;
        enableDisguise = enableDisguise * 1;

        // todo move to common file
        // eslint-disable-next-line
        tmpurl = format('ftp.csp?ftphost={0}{1}&ftpport={2}', host, enableDisguise ? '&safeurl=1' : '', port);
        tmpurl = 'https://' + window.location.hostname + (!window.location.port ? '' : ':' + window.location.port) +
            '/por/' + tmpurl + '&TWFID=' + session.getTWFID();

        realurl = tmpurl;

        //add by cfq for Bug 9020,2009-11-27
        try {
            if (resConfig.sso.data.length > 0 && _isCanUseSSO(resConfig.sso.data[rcid])) {
                realurl = realurl + '?' + resConfig.sso.obj.GetSSOLoginString;
            }
        } catch (e) {
            SFLOG.info(LOGTAG, '_handleFtp() catch error: ' + e.message);
        }

        if (isDefault && (resConfig.rc.defaults.rcId > 0)) {

            if (
                (typeof(rcid) !== 'undefined') &&
                (resConfig.sso.data.length > 0) &&
                _isCanUseSSO(resConfig.sso.data[rcid])
            ) {

                realurl = _getSsoParams(_getRcById(rcid), realurl);
                SFCommon.openWindow({url: realurl, target: !isDefault});

                // resConfig.sso.obj.CreateIEWithHandle(realurl, '');
                return;
            }

            SFCommon.openWindow({url: realurl, target: !isDefault});
        } else {
            SFCommon.openWindow({url: realurl});
        }
    }

    function _getSsoParams (rsItem, url) {
        var id = rsItem.id,
            vpnURL = window.location.protocol + '//' + window.location.host,
            twfid = session.getTWFID(),
            ecport = cacheParam.port,
            ecguid = cacheParam.guid,
            ectoken = cacheParam.token,
            ssoAddrURL;

        if (
            com.isWeb() ||  // 浏览器所有资源都通过VPN地址跳转
            !SFConfig.isExistEC ||  // 无EC, 都通过VPN地址跳转
            (rsItem.type === rcType.WEB && !resConfig.rc.easyLinkHash[id])  // esaylink因为跨域, 优先用local跳转
        ) {
            ssoAddrURL = vpnURL + '/portal/sso.html';
        } else {
            ssoAddrURL = 'https://127.0.0.1:' + ecport + '/Web/sso.html';
        }

        url = encodeURIComponent(url);
        return ssoAddrURL + '?id=' + id + '&url=' + url + '&twfid=' + twfid + '&ecport=' + ecport + '&ecguid=' + ecguid + '&ectoken=' + ectoken;
    }

    // 处理mail资源
    function _handleMail (enableDisguise, host, port, domain, isDefault, rcid) {
        var tmpurl,
            realurl;
        enableDisguise = enableDisguise * 1;
        tmpurl = format('mail.csp?smtphost={0}&imapport={1}&domain={2}{3}', host, port, domain, enableDisguise ? '&safeurl=1' : '');

        tmpurl = 'https://' + window.location.hostname + (!window.location.port ? '' : ':' + window.location.port) +
            '/por/' + tmpurl + '&TWFID=' + session.getTWFID();
        realurl = tmpurl;
        try {
            if ((resConfig.sso.data.length > 0) && _isCanUseSSO(resConfig.sso.data[rcid])) {
                realurl = realurl + '?' + resConfig.sso.obj.GetSSOLoginString;
            }
        } catch (e) {
            SFLOG.info(LOGTAG, '_handleMail() catch error: ' + e.message);
        }

        if (
            (isDefault && resConfig.rc.defaults.rcId > 0) &&
            ((typeof (rcid) !== 'undefined') && (resConfig.sso.data.length > 0) && _isCanUseSSO(resConfig.sso.data[rcid]))
        ) {
            realurl = _getSsoParams(_getRcById(rcid), realurl);
            SFCommon.openWindow({url: realurl, target: !isDefault});

            // resConfig.sso.obj.CreateIEWithHandle(realurl, '');
        } else {
            SFCommon.openWindow({url: realurl});
        }
    }

    // 打开web资源
    function _openWebResource (rcInfo, url, isEasyLink, isSSO, isDefault) {
        var isEC = !com.isWeb(),
            targetURL;

        if (isSSO) {
            targetURL = _getSsoParams(rcInfo, url);

        } else if (!isEasyLink && isEC) {

            // EC 中需要打开中转页面设置会话
            targetURL = buildShortcutURL(url);

        } else {
            targetURL = url;
        }
        if (_isCanUseSSO(resConfig.sso.data[rcInfo.rcId])) {
            SFCommon.openWindow({url: targetURL, target: !isDefault, browser: window.browserTypes.ie});
        } else {
            SFCommon.openWindow({url: targetURL, target: !isDefault});
        }

    }


    // 构造中转页面URL
    function buildShortcutURL (url) {
        return   cacheParam.vpnURL + '/portal/shortcut.html?twfid=' + session.getTWFID() + '&url=' + encodeURIComponent(url);
    }

    function _format (date, format) {
        var o = {
                'M+': date.getMonth() + 1,		//月
                'd+': date.getDate(),     		//日
                'H+': date.getHours(),    		//时
                'm+': date.getMinutes(),  		//分
                's+': date.getSeconds(),  		//秒
                'S': date.getMilliseconds()     //毫秒
            },
            k,
            yearLen = 4;
        if (/(y+)/.test(format)) {
            format = format.replace(RegExp.$1, (date.getFullYear() + '').substr(yearLen - RegExp.$1.length));
        }
        for (k in o) {
            if (o.hasOwnProperty(k)) {
                if (new RegExp('(' + k + ')').test(format)) {
                    format = format.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
                }
            }
        }
        return format;

    }

    // eslint-disable-next-line
    /**
     * 构造数组
     * 从xml转换为json的数据中单条数据和空数据需要特殊处理
     * path 路径: 'Resource.rcs'
     * node 数据节点
     * @returns {Array}
     */
    function _buildArray (path, node) {
        var subNode,
            arr = [];
        subNode = _getNodeValue(path, [], node);
        if (is.array(subNode)) {
            arr = subNode;
        } else {
            arr.push(subNode);
        }
        return arr;
    }

    // 资源ip解析
    function _buildIpArray (ipStr) {
        var ips,
            ip,
            regons;
        ips = ipStr.split(';');
        ips.forEach(function (value, i) {
            regons = value.split('~');
            ip = {};
            ip.host_from = regons[0];
            ip.host_to = is.set(regons[1]) ? regons[1] : '';
            ips[i] = ip;
        });
        return ips;
    }


    // 资源端口解析
    function _buildPortArray (portStr) {
        var ports, port, regons;
        ports = portStr.split(';');
        ports.forEach(function (value, i) {
            regons = value.split('~');
            port = {};
            port.port_from = regons[0];
            port.port_to = is.set(regons[1]) ? regons[1] : regons[0];
            ports[i] = port;
        });
        return ports;
    }

    // 解码HTML特殊标签
    function _htmlDecode (input) {
        var output = input.toString();
        output = output.replace(/&quot;/g, '\'');
        output = output.replace(/&apos;/g, '');
        output = output.replace(/&lt;/g, '<');
        output = output.replace(/&gt;/g, '>');
        output = output.replace(/&amp;/g, '&');
        output = output.replace(/&nbsp;/g, ' ');
        return output;
    }

    function _parseUrl (url) {
        var a = document.createElement('a');
        a.href = url || location.href;
        return {
            source: url,
            protocol: a.protocol.replace(':', ''),
            host: a.hostname,
            port: (function () {
                if (a.port && a.port > 0) {
                    return a.port;
                } 
                return a.protocol === 'https:' ? '443' : '80';
                
            })(),
            query: a.search,
            params: (function () {
                var ret = {},
                    seg = a.search.replace(/^\?/, '').split('&'),
                    len = seg.length,
                    i = 0,
                    s;
                for (; i < len; i++) {
                    if (!seg[i]) {
                        continue;
                    }
                    s = seg[i].split('=');
                    ret[s[0]] = s[1];
                }
                return ret;
            })(),
            file: (a.pathname.match(/\/([^\/?#]+)$/i) || [, ''])[1],
            hash: a.hash.replace('#', ''),
            path: a.pathname.replace(/^([^\/])/, '/$1'),
            relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || [, ''])[1],
            segments: a.pathname.replace(/^\//, '').split('/')
        };
    }


    // SFAPI.doConfigEC({
    //     key: '',
    //     params: {

    //     },
    //     success: function () {

    //     },
    //     error: function () {

    //     }
    // });

    SF.res = resObj = new Res;
    SF.resConfig = resConfig;

})(this, SF);